#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <wfdb/wfdb.h>
#include <wfdb/ecgmap.h>
#include <wfdb/ecgcodes.h>

void filt(double *a, long n, char isign) {
   double C0=0.4829629131445341, C1=0.8365163037378077, C2=0.2241438680420134, C3=-0.1294095225512603;
   double *wksp;
   long nh,i,j;
   if (n < 4) return;
   wksp = (double *)malloc(n*sizeof(double));
   nh = n >> 1;
   if (isign >= 0) {
      for (i=0,j=0;j<n-3;j+=2,i++) {
         wksp[i] = C0*a[j]+C1*a[j+1]+C2*a[j+2]+C3*a[j+3];
         wksp[i+nh] = C3*a[j]-C2*a[j+1]+C1*a[j+2]-C0*a[j+3];
      }
      wksp[i] = C0*a[n-2]+C1*a[n-1]+C2*a[0]+C3*a[1];
      wksp[i+nh] = C3*a[n-2]-C2*a[n-1]+C1*a[0]-C0*a[1];
   } else {
      wksp[0] = C2*a[nh-1]+C1*a[n-1]+C0*a[0]+C3*a[nh];
      wksp[1] = C3*a[nh-1]-C0*a[n-1]+C1*a[0]-C2*a[nh];
      for (i=0,j=2;i<nh-1;i++) {
         wksp[j++] = C2*a[i]+C1*a[i+nh]+C0*a[i+1]+C3*a[i+nh+1];
         wksp[j++] = C3*a[i]-C0*a[i+nh]+C1*a[i+1]-C2*a[i+nh+1];
      }
   }
   for (i=0;i<n;i++) a[i]=wksp[i];
   free((void *)wksp);
}

void condition(double *a, long n, char isign) {
   double t0,t1,t2,t3;
   if (n < 4) return;
   if (isign >= 0) {
      t0 = 0.324894048898962*a[0]+0.0371580151158803*a[1];
      t1 = 1.00144540498130*a[1];
      t2 = 1.08984305289504*a[n-2];
      t3 = -0.800813234246437*a[n-2]+2.09629288435324*a[n-1];
      a[0]=t0; a[1]=t1; a[n-2]=t2; a[n-1]=t3;
   } else {
      t0 = 3.07792649138669*a[0]-0.114204567242137*a[1];
      t1 = 0.998556681198888*a[1];
      t2 = 0.917563310922261*a[n-2];
      t3 = 0.350522032550918*a[n-2]+0.477032578540915*a[n-1];
      a[0]=t0; a[1]=t1; a[n-2]=t2; a[n-1]=t3;
   }
}

void wt1(double *a, long n, char isign)
{
   long nn;
   if (n < 4) return;
   if (isign >= 0) {
      condition(a,n,1);
      for (nn=n;nn>=4;nn>>=1) filt(a,nn,isign);
   } else {
      for (nn=4;nn<=n;nn<<=1) filt(a,nn,isign);
      condition(a,n,-1);
   }
}

char *trim_whitespace(char *str)
{
   size_t len = 0;
   char *frontp = str - 1;
   char *endp = NULL;

   if (str == NULL) return NULL;
   if (str[0] == '\0') return str;
   len = strlen(str);
   endp = str + len;
   while (isspace(*(++frontp)));
   while (isspace(*(--endp)) && endp != frontp);
   if (str + len - 1 != endp)
      *(endp + 1) = '\0';
   else if (frontp != str && endp == frontp)
      *str = '\0';
   endp = str;
   if (frontp != str) {
      while (*frontp) *endp++ = *frontp++;
      *endp = '\0';
   }
   return str;
}

main(argc, argv)
int argc;
char *argv[];
{
   char zero, flag, *record = NULL;
   short refract, refract2;
   int sig, nsig;
   double *sv, aux, approximationLevel, log2();
   float max, summax, med;
   long i, j, nsv, nmax, idealLen, begin;
   WFDB_Anninfo a;
   WFDB_Annotation annot;
   WFDB_Siginfo *s;
   WFDB_Sample v[WFDB_MAXSIG];
   WFDB_Frequency freq;

   if(argc < 2) {
      fprintf(stderr, "usage: %s record\n", argv[0]);
      exit(1);
   }

   record = argv[1];
   if ((nsig = isigopen(record, NULL, 0)) < 1) exit(2);
   if ((s = (WFDB_Siginfo *)malloc(nsig * sizeof(WFDB_Siginfo))) == NULL) {
      (void)fprintf(stderr, "insufficient memory\n");
      exit(2);
   }
   a.name = "sv"; a.stat = WFDB_WRITE;
   if ((nsig = wfdbinit(record, &a, 1, s, nsig)) < 1) exit(2);

   /* Identify the lowest-numbered SV signal */
   for (i = 0; i < nsig; i++)
      if (strcmp(trim_whitespace(s[i].desc), "SV") == 0)
            break;
   if (i == nsig) {
      fprintf(stderr, "No SV signal specified\n");
      exit(3);
   }
   sig = i;

// Frecuencia de muestreo
   freq = sampfreq(argv[1]);
   refract = 0.2*freq;
   refract2 = refract/2;

   nsv = s[sig].nsamp;

   idealLen = (long)pow(2, ( ceil( log2((double)nsv) ) ) );
   sv = (double *)malloc(idealLen*sizeof(double));

   for(i=0; i< nsv; i++) {
      getvec(v);
      sv[i] = v[sig];
   }

//   wt1(sv, nsv, 1);
//   wt1(sv, nsv, -1);


   wt1(sv, idealLen, 1);

   approximationLevel = 5;

   begin = (long) (idealLen / pow(2.0, approximationLevel));

   for (i = begin; i < idealLen; i++) {
      sv[i] = 0.0;
   }

   wt1(sv, idealLen, -1);





// Inicializamos la estructura de anotaciones. Todos los latidos se etiquetan como normales
   annot.anntyp = NORMAL;
   annot.subtyp = 0;
   annot.chan = 0;
   annot.num = 0;
   zero = 0;
   annot.aux = &zero;

   summax = 0;
   nmax = 0;

   for(i=1; i<nsv-1; i++) {
      flag = 1;
      j = i-refract2;
      if(j<0) j=0;
      while(j<i+refract2 && j<nsv) {
         if(sv[i] > sv[j]) {
            flag = 0;
            break;
         }
         j++;
      }
      if(flag) {
         j = i - freq;
         if(j<0) j=0;
         max = sv[j]; j++;
         while(j<i+freq && j<nsv) {
            if(sv[j] > max) max = sv[j];
            j++;
         }
         summax+= max - sv[i];
         nmax++;
      }
   }

   med = summax/nmax;
//printf("med = %f\n", med);
   for(i=1; i<nsv-1; i++) {
      flag = 1;
      j = i-refract2;
      if(j<0) j=0;
      while(j<i+refract2 && j<nsv) {
         if(sv[i] > sv[j]) {
            flag = 0;
            break;
         }
         j++;
      }
      if(flag) {
         j = i - freq;
         if(j<0) j=0;
         max = sv[j]; j++;
         while(j<i+freq && j<nsv) {
            if(sv[j] > max) max = sv[j];
            j++;
         }
      }
      if(flag && (max-sv[i]) > 0.7*med) {
//if(flag) {
//         printf("%ld %f", i, max - sv[i]);
//if((max - sv[i]) < 0.7*med) printf(" *");
//printf("\n");
         annot.time = i;
         putann(0, &annot);
         i += refract;
      }
   }
   free((void *)sv);
   wfdbquit();
}
